*============================================================================
* mcmcrun:   Housekeeping program for running user written MCMC analyses
* Author: John Thompson
* Date:   Dec 2012
*============================================================================

program mcmcrun
   version 12.1
   syntax anything [using/] [fweight] [if] [in] , /// using gives the name of the output file, anything -> logpost b
   Samplers(string)                             /// Samplers for use in the run
   [                                            ///
   Burnin(integer 0)                            /// Length of the burnin
   Updates(integer 1000)                        /// Number of updates in the run
   ADAPT                                        /// Adapt the samplers during the burnin
   PArameters(string)                           /// Parameter names
   PRedictions(string)                          /// functions that makes predictions
   REPlace                                      /// replace existing output file
   APPend                                       /// append to existing output file
   JPosterior                                   /// do not recalculate the current log-posterior at each step
   SAVELOGP                                     /// save log-posterior in the output file
   DOTs(integer 250)                            /// interval between dots in the results window
   MATA                                         /// User functions written in Mata
   Data(string)                                 /// data variables needed for calculating the log-posterior
   Thin(integer 1)                              /// thin the chain before saving
   ]
   
   if "`mata'" != "" mcmcrun_mata `0'
   else mcmcrun_stata `0'
end

*-----------------------------------
* User's functions written in Stata
*-----------------------------------
program mcmcrun_stata , rclass
   syntax anything [using/] [fweight] [if] [in] , ///
   Samplers(string) [ ///
   Burnin(integer 0) ///
   Updates(integer 1000) ///
   ADAPT ///
   PArameters(string) ///
   PRedictions(string)  ///
   REPlace ///
   APPend ///
   JPosterior ///
   SAVELOGP ///
   DOTs(integer 250) ///
   Thin(integer 1) ]
   
*----------------------------
* Interpret anything
*----------------------------
   tokenize `"`anything'"'
   local logpost "`1'"
   local b "`2'"
   local filename "`using'"
   
*-----------------------------------------------------------------------------------
* Interpret the samplers: store name of sampler(smp`k') and its parameters(prm`k')
*-----------------------------------------------------------------------------------
   tokenize `"`samplers'"' , parse(" (),")
   local i = 1
   local k = 0
   while "``i''" != "" {
*--------------------------
* Same sampler used m times
*--------------------------
      local j = `i' + 1
      if "``j''" == "(" {
         local m = ``i''
         local i = `i' + 1
      }
      else local m = 1
*--------------------------
* Samplers maybe in ()
*--------------------------
      if "``i''" == "(" {
         local inbr = 1
         local ++i
      }
      else local inbr = 0
*-----------------------------------
* Allow either mhsnorm or just norm
*-----------------------------------
      if inlist("``i''","norm","logn","trnc","mnorm") local s "mhs``i''"
      else if inlist("``i''","slice","griddy","arms","ars") local s "gbs``i''"
      else local s "``i''"
      local ++i
*---------------------------------------
* Read options
*---------------------------------------
      local br = 1
      local p ""
      while `br' != 0 & `"``i''"' != "" {
         if `"``i''"' == ")" local --br
         else if `"``i''"' == "(" local ++br
         if `br' != 0 {
            if `"``i''"' != "("   local p `"`p' ``i''"'
            else local p `"`p'``i''"'
         }
         local ++i
      }
      if (`inbr' == 1 & `br' != 0) | (`inbr' == 0 & `br' != 1 ) {
         di as err "bracketing error for sampler `s'"
         exit(132)
      }
      forvalues j = 1/`m' {
         local ++k
         local smp`k' "`s'"
         local prm`k' `"`p'"'
      }
   }
   local nSamplers = `k'
*----------------------------
* Check if jpostuation is set
*----------------------------
   if "`jposterior'" != "" {
      local jpost "logp(.)"
   }
*----------------------------
* check for weights & ifin
*----------------------------
   local d = 1
   local condition "`if' `in'"
   if "`weight'" != "" local condition "[`weight'`exp'] `condition'"
*------------------------------------------------------
* Create the command cmd`p' for running each sampler
* by combining smp`p' & prm`p' & inserting logpost & b
*------------------------------------------------------
   forvalues p = 1/`nSamplers' {
      local sd`p' ""
      local allowlogp`p' "N"
*---------------------------
* STANDARD SAMPLER
*---------------------------
      if inlist("`smp`p''","mhsnorm","mhslogn","mhstrnc","mhsmnorm", ///
      "gbsgriddy","gbsarms","gbsars","gbsslice") {
*---------------------------
* Parse the parameters
*---------------------------
         local 0 `"`prm`p''"'
         syntax , [ SD(string) Lbound(string) Ubound(string) MAXtry(string) Grid(string) ///
         Cholesky(string) Metropolis Histogram Stepsize(string) Nsteps(string) X(string) DEBUG]
         local lb = cond("`lbound'" == "","","lb(`lbound')")
         local ub = cond("`ubound'" == "","","ub(`ubound')")
         local mt = cond("`maxtry'" == "","","max(`maxtry')")
         local st = cond("`stepsize'" == "","","step(`stepsize')")
         local ns = cond("`nsteps'" == "","","nstep(`nsteps')")
         local X = cond("`x'" == "","X","`x'")
*-------------------------------
* SD may be scalar or in matrix
*-------------------------------
         if "`sd'" != "" {
            tokenize "`sd'" , parse(" [")
            cap confirm matrix `1'
            if !_rc {
               if "`2'" != "[" {
                  local sd "`sd'[1,`d']"
               }
               if "`adapt'" != "" local sd`p' "matrix `sd' = `sd'"
            }
            cap confirm scalar `1'
            if !_rc {
               if "`adapt'" != "" local sd`p' "scalar `sd' = scalar(`sd')"
               local sd "scalar(`sd')"
            }
         }
*---------------------
* M-H samplers
*---------------------
            if "`smp`p''" == "mhsnorm" | "`smp`p''" == "mhslogn"{
               local cmd`p' "`smp`p'' `logpost' `b' `d' `condition' , sd(`sd') `debug'"
               local ++d
               local size`p' = 1
               local allowlogp`p' "Y"
            }
            else if "`smp`p''" == "mhsmnorm" {
               local cmd`p' "`smp`p'' `logpost' `b' `d' `condition' , cholesky(`cholesky') `debug'"
               local size`p' = rowsof(`cholesky')
               local d = `d' + `size`p''
               local allowlogp`p' "Y"
            }
            else if "`smp`p''" == "mhstrnc" {
               local cmd`p' "`smp`p'' `logpost' `b' `d' `condition' , sd(`sd') `lb' `ub' `debug'"
               local ++d
               local size`p' = 1
               local allowlogp`p' "Y"
            }
*---------------------
* Griddy sampler
*---------------------
         else if "`smp`p''" == "gbsgriddy" {
               local g ""
               foreach j of numlist `grid' {
                  local g "`g' `j'"
               }
               local cmd`p' "`smp`p'' `logpost' `b' `d' `condition' , grid(`g')  `histogram' `metropolis'"
               local size`p' = 1
               local ++d
               local allowlogp`p' "Y"
         }
*---------------------
* ARMS sampler
*---------------------
         else if "`smp`p''" == "gbsarms" {
               local g ""
               foreach j of numlist `grid' {
                  local g "`g' `j'"
               }
               local cmd`p' "`smp`p'' `logpost' `b' `d' `condition' , grid(`g') `lb' `ub' `mt'"
            local size`p' = 1
            local ++d
               local allowlogp`p' "Y"
         }
*---------------------
* ARS sampler
*---------------------
         else if "`smp`p''" == "gbsars" {
               local g ""
               foreach j of numlist `grid' {
                  local g "`g' `j'"
               }
               local cmd`p' "`smp`p'' `logpost' `b' `d' `condition' , grid(`g') `lb' `ub' `mt'"
            local size`p' = 1
            local ++d
         }
*---------------------
* Slice sampler
*---------------------
         else if "`smp`p''" == "gbsslice" {
            if "`maxtry'" == "" local maxtry = 50
            if "`stepsize'" == "" local stepsize = 1
            if "`nsteps'" == "" local nsteps = 6
            local cmd`p' "`smp`p'' `logpost' `b' `d' `condition' , `lb' `ub' `st' `ns' `mt'"
            local size`p' = 1
            local ++d
            local allowlogp`p' "Y"
         }
      }
*---------------------------
* Skip
*---------------------------
      else if "`smp`p''" == "skip" {
	      local cmd`p' "`smp`p''"
          local size`p' = 1
          local ++d
	  }
*---------------------------
* USER's OWN SAMPLER
*---------------------------
      else {
         local cmd`p' "`smp`p'' `logpost' `b' `d' `condition' "
         local 0 `"`prm`p''"'
         syntax [anything ] , [ * Dim(integer 1) ]
         local size`p' = `dim'
         local cmd`p' `"`cmd`p'' , `options'"'
         local d = `d' + `size`p''
         local allowlogp`p' "N"
         local sd`p' ""
      }
   }
   local npar = `d' - 1
*-----------------------------------
* Open file for results
*-----------------------------------
   if "`filename'" != "" {
      tempname fh
      file open `fh' using `filename' , `replace' `append' write
*------------------------------------------
* write file header line if not appending
*------------------------------------------
      if "`append'" == "" {
*------------------------------
* saving logp?
*------------------------------
         if "`savelogp'" != "" file write `fh' "logp,"
*--------------------------------------
* names for parameters if unnamed by user
* call them p1,p2,..
*--------------------------------------
         if "`parameters'" == "" {
            file write `fh' "p1"
            forvalues p = 2/`npar' {
               file write `fh' ",p`p'"
            }
         }
*----------------------------------
* interpret user's given parameter names
* a list or perhaps x1-x7
*----------------------------------
         else {
            tokenize "`parameters'" , parse(" -")
            file write `fh' "`1'"
            local pm1 "`1'"
            local i = 2
            local h = 1
            while "``i''" != "" {
               if "``i''" == "-" {
                  local j = `i' - 1
                  local t1 "``j''"
                  local ++i
                  local t2 "``i''"
                  local j = strlen("`t1'")
                  while real(substr("`t1'",`j',.)) != . {
                     local --j
                  }
                  local base = substr("`t1'",1,`j')
                  local v1 = substr("`t1'",`j'+1,.)
                  local ++v1
                  local v2 = substr("`t2'",`j'+1,.)
                  
                  forvalues j = `v1'/`v2' {
                     file write `fh' ",`base'`j'"
                     local ++h
                     local pm`h' "`base'`j'"
                  }
               }
               else {
                  file write `fh' ",``i''"
                  local ++h
                  local pm`h' "``i''"
               }
               local ++i
            }
		    if `h' != `npar' {
		       di as err "`h' parameter names but `npar' parameters referenced by samplers"
			   file close `fh'
			   exit(499)
		    }
         }
*----------------------------------
* names of any predictions
*----------------------------------
         if `"`predictions'"' != "" {
		    `predictions' `b'
			tempname npd
			matrix `npd' = r(pred)
			local npredictions = colsof(`npd')
            forvalues j = 1/`npredictions' {
               file write `fh' ",`predictions'`j'"
            }
         }
         file write `fh' _n
      }
   }
*------------------------------------
* Display choices to results window
*------------------------------------
   di as txt " MCMC using user program `logpost'"
   di as txt " `npar' parameters"
   di as txt " Initial values "
   forvalues j = 1/`npar' {
      di as res %10.4f `b'[1,`j'] _continue
      if mod(`j',5) == 0 di as txt " "
   }
   if mod(`npar',5) != 0 di as txt " "
   di as txt "Samplers"
   local i = 1
   forvalues j = 1/`nSamplers' {
      local ii = `i' - 1 + `size`j''
      if "`parameters'" == "" | "`filename'" == "" {
         if `i' == `ii' di as txt %4.0f `i' %5s " " _continue
         else di as txt %4.0f `i' "-" %4.0f `ii' _continue
      }
      else {
         if `i' == `ii' di as txt %6s "`pm`i''" %7s " " _continue
         else di as txt %6s "`pm`i''" "-" %6s "`pm`ii''" _continue
      }
      di as res " " %-50s "`cmd`j''"
      local i = `ii' + 1
   }
   di as txt "Length of Burn-in = `burnin'"
   if `burnin' > 0 {
      if "`adapt'" != "" di as txt "Adapting sd during the burn-in"
      if `dots' > 0 di as txt "dots displayed every `dots' iterations"
   }
*-------------------------------------
* Run the burn-in - results not saved
*-------------------------------------
   tempname ipar
   timer clear 2
   timer on 2
   forvalues iter = 1/`burnin' {
      if `dots' > 0 {
         if mod(`iter',`dots') == 0 di  "." _continue
      }
      scalar `ipar' = 1
      forvalues p = 1/`nSamplers' {
*--------------------------------------------
* if sampler=skip do nothing otherwise run
*--------------------------------------------
         if "`smp`p''" != "skip"  {
             if "`allowlogp`p''" == "Y" `cmd`p'' `jpost'
             else `cmd`p''
         }
*--------------------------------------------
* Save the returned logp
*--------------------------------------------
         local slp = r(logp)
         if "`jposterior'" != "" local jpost "logp(`slp')"
*--------------------------------------------
* Adapt the standard deviation
*--------------------------------------------
         if "`sd`p''" != "" {
            if r(accept) == 1 `sd`p''*1.01
            else `sd`p''/1.01
         }
         scalar `ipar' = `ipar' + `size`p''
      }
   }
*-----------------------------------------------
* Display state after burn-in to results window
*-----------------------------------------------
   timer off 2
   qui timer list 2
   return scalar burnintime = r(t2)
   di as txt " "
   if `burnin' != 0 {
      di as txt "Burn-in completed in " %10.3f `r(t2)' " seconds"
      return scalar burnintime = r(t2)
      return scalar burnin = `burnin'
      di as txt "Parameter values after burn-in"
      forvalues j = 1/`npar' {
         di as res %10.4f `b'[1,`j'] _continue
         if mod(`j',5) == 0 di as txt " "
      }
      if mod(`npar',5) != 0 di as txt " "
   }
   di as txt "Run of length `updates' has started"
   if `"`filename'"' != "" & "`append'" == "" di as txt "Run saved to `filename'"
   if `"`filename'"' != "" & "`append'" != "" di as txt "Run appended to `filename'"
   if `"`predictions'"' != "" di as txt "Predictions stored as `predictions'1, `predictions'2, ..."
   if `dots' > 0 di as txt "dots displayed every `dots' iterations"
   timer clear 2
   timer on 2
*----------------------------
* Run updates - saving results
*----------------------------
   tempname mpred
   forvalues iter = 1/`updates' {
      if `dots' > 0 {
         if mod(`iter',`dots') == 0 di  "." _continue
      }
      scalar `ipar' = 1
      forvalues p = 1/`nSamplers' {
*--------------------------------------------
* if sampler=skip do nothing otherwise run
*--------------------------------------------
         if "`smp`p''" != "skip"  {
             if "`allowlogp`p''" == "Y" `cmd`p'' `jpost'
             else `cmd`p''
         }
*--------------------------------------------
* Save the returned logp
*--------------------------------------------
         local slp = r(logp)
         if "`jposterior'" != "" local jpost "logp(`slp')"
         scalar `ipar' = `ipar' + `size`p''
      }
*------------------------------------
* Write results to output file
*------------------------------------
      if "`filename'" != "" & mod(`iter',`thin') == 0 {
         if "`savelogp'" != "" file write `fh' "`slp',"
         local a = `b'[1,1]
         file write `fh' "`a'"
         forvalues p = 2/`npar' {
            local a = `b'[1,`p']
            file write `fh' ",`a'"
         }
*-----------------------
* calculate predictions
*-----------------------
         if `"`predictions'"' != "" {
            `predictions' `b'
            matrix `mpred' = r(pred)
            forvalues p=1/`npredictions' {
               local a = `mpred'[1,`p']
               file write `fh' ",`a'"
            }
         }
         file write `fh' _n
      }
   }
*----------------------------
* Display final report
*----------------------------
   timer off 2
   qui timer list 2
   di as txt " "
   di as txt "Run completed in " %10.3f `r(t2)' " seconds"
   return scalar updatetime = r(t2)
*----------------------------
* finish off
*----------------------------
   if "`filename'" != "" {
      file close `fh'
   }
end


program mcmcrun_mata , rclass
   syntax anything [using/] [fweight] [if] [in] , ///
   Samplers(string) [ ///
   Burnin(integer 0) ///
   Updates(integer 1000) ///
   ADAPT ///
   PArameters(string) ///
   PRedictions(string)  ///
   REPlace ///
   JPosterior ///
   SAVELOGP ///
   DOTs(integer 250) ///
   MATA ///
   Data(string) ///
   Thin(integer 1) ]
*----------------------------
* Interpret anything
*----------------------------
   tokenize `"`anything'"'
   local logpost "`1'"
   local X "`2'"
   local b "`3'"
   local filename `"`using'"'
*-----------------------------------------------------------------------------------
* Interpret the samplers: store name of sampler(smp`k') and its parameters(prm`k')
*-----------------------------------------------------------------------------------
   tokenize `"`samplers'"' , parse(" *(),")
   local i = 1
   local k = 0
   while "``i''" != "" {
*--------------------------
* Same sampler used m times
*--------------------------
      local j = `i' + 1
      if "``j''" == "(" {
         local m = ``i''
         local i = `i' + 1
      }
      else local m = 1
*--------------------------
* Samplers maybe in ()
*--------------------------
      if "``i''" == "(" {
         local inbr = 1
         local ++i
      }
      else local inbr = 0
*-----------------------------------
* Allow either mhsnorm or just norm
*-----------------------------------
      if inlist("``i''","norm","logn","trnc","mnorm") local s "mhs``i''"
      else if inlist("``i''","slice","griddy","arms","ars") local s "gbs``i''"
      else local s "``i''"
      local ++i
*---------------------------------------
* Read options
*---------------------------------------
      local br = 1
      local p ""
      while `br' != 0 & `"``i''"' != "" {
         if `"``i''"' == ")" local --br
         else if `"``i''"' == "(" local ++br
         if `br' != 0 {
            if `"``i''"' != "("   local p `"`p' ``i''"'
            else local p `"`p'``i''"'
         }
         local ++i
      }
      if (`inbr' == 1 & `br' != 0) | (`inbr' == 0 & `br' != 1 ) {
         di as err "bracketing error for sampler `s'"
         exit(132)
      }
      forvalues j = 1/`m' {
         local ++k
         local smp`k' "`s'"
         local prm`k' `"`p'"'
      }
   }
   local nSamplers = `k'
*----------------------------
* Check if norecaluation is set
*----------------------------
   if "`jposterior'" != "" {
      local jpost "logp(.)"
   }
*----------------------------
* check for weights & ifin
*----------------------------
   local d = 1
   local condition "`if' `in'"
   if "`weight'" != "" local condition "[`weight'`exp'] `condition'"
*----------------------------
* Create the command
*----------------------------
   forvalues p = 1/`nSamplers' {
      local sd`p' ""
      if inlist(`"`smp`p''"',"mhsnorm","mhslogn","mhstrnc","mhsmnorm", ///
      "gbsgriddy","gbsarms","gbsars","gbsslice") {
*----------------------------
* Parse the samplers
*----------------------------
         local 0 `"`prm`p''"'
         syntax , [ SD(string) LBound(string) UBound(string) MAXtry(string) Grid(string) ///
         Cholesky(string) Npoints(string) METropolis HIStogram STepsize(string) NSteps(string) X(string)]
         local lbound = cond("`lbound'" == "",".","`lbound'")
         local ubound = cond("`ubound'" == "",".","`ubound'")
		 local sd`p' ""
         if "`sd'" != "" {
            tokenize "`sd'" , parse(" [")
            cap confirm matrix `1'
            if !_rc {
               if "`2'" != "[" {
                  local sd "`sd'[`d']"
               }
               if "`adapt'" != "" local sd`p' "`sd' = `sd'"
            }
			else {
               cap confirm scalar `1'
               if !_rc & "`adapt'" != "" local sd`p' " `sd' = `sd'"
			}
         }
*---------------------------
* MH sampler
*---------------------------
         if inlist("`smp`p''","mhsnorm","mhslogn","mhstrnc","mhsmnorm") {            
            if "`smp`p''" == "mhsmnorm" {
               if "`jposterior'" != "" local cmd`p' "`smp`p''(logp,&`logpost'(),`X',`b',`d',`cholesky',accept)"
               else local cmd`p' "`smp`p''(logp=.,&`logpost'(),`X',`b',`d',`cholesky',accept)"
               local size`p' = rowsof(`cholesky')
               local d = `d' + `size`p''
            }
            else if "`smp`p''" == "mhstrnc" {
               if "`jposterior'" != "" local cmd`p' "`smp`p''(logp,&`logpost'(),`X',`b',`d',`sd',`lbound',`ubound',accept)"
               else local cmd`p' "`smp`p''(logp=.,&`logpost'(),`X',`b',`d',`sd',`lbound',`ubound',accept)"
               local ++d
               local size`p' = 1
            }
            else {
               if "`jposterior'" != "" local cmd`p' "`smp`p''(logp,&`logpost'(),`X',`b',`d',`sd',accept)"
               else local cmd`p' "`smp`p''(logp=.,&`logpost'(),`X',`b',`d',`sd',accept)"
               local ++d
               local size`p' = 1
            }
         }
*---------------------------
* Griddy Sampler
*---------------------------
         else if "`smp`p''" == "gbsgriddy" {
            local metropolis = cond("`metropolis'" == "","0","1")
            local histogram = cond("`histogram'" == "","0","1")
            if "`grid'" == "" {
               local npoints = cond("`npoints'" == "","10","`npoints'")
               local g "`lbound'"
               local np1 = `npoints' - 1
               forvalue j=2/`np1' {
                  local g "`g' . "
               }
               local g "`g' `ubound'"
            }
            else {
               local g ""
               foreach j of numlist `grid' {
                  local g "`g' `j'"
               }
            }
            if "`jposterior'" != "" local cmd`p' `"`smp`p''(logp,&`logpost'(),`X',`b',`d',"`g'",`metropolis',`histogram')"'
            else local cmd`p' `"`smp`p''(logp=.,&`logpost'(),`X',`b',`d',"`g'",`metropolis',`histogram')"'
            local size`p' = 1
            local ++d
         }
*---------------------------
* Slice sampler
*---------------------------
         else if "`smp`p''" == "gbsslice" {
            if "`maxtry'" == "" local maxtry = 50
            if "`stepsize'" == "" local stepsize = 1
            if "`nsteps'" == "" local nsteps = 6
            if "`jposterior'" != "" local cmd`p' "`smp`p''(logp,&`logpost'(),`X',`b',`d',`stepsize',`nsteps',`lbound',`ubound',`maxtry')"
            else local cmd`p' "`smp`p''(logp=.,&`logpost'(),`X',`b',`d',`stepsize',`nsteps',`lbound',`ubound',`maxtry')"
            local size`p' = 1
            local ++d
         }
*---------------------------
* ARMS sampler
*---------------------------
         else if "`smp`p''" == "gbsarms" {
            if "`maxtry'" == "" local maxtry = 50
            if "`grid'" == "" {
               local npoints = cond("`npoints'" == "","10","`npoints'")
               local g "`lbound'"
               local np1 = `npoints' - 1
               forvalue j=2/`np1' {
                  local g "`g' . "
               }
               local g "`g' `ubound'"
            }
            else {
               local g ""
               foreach j of numlist `grid' {
                  local g "`g' `j'"
               }
            }
            if "`jposterior'" != "" local cmd`p' `"`smp`p''(logp,&`logpost'(),`X',`b',`d',"`g'",`maxtry')"'
            else local cmd`p' `"`smp`p''(logp=.,&`logpost'(),`X',`b',`d',"`g'",`maxtry')"'
            local size`p' = 1
            local ++d
         }
*---------------------------
* ARS sampler
*---------------------------
         else if "`smp`p''" == "gbsars" {
            if "`maxtry'" == "" local maxtry = 50
            if "`grid'" == "" {
               local npoints = cond("`npoints'" == "","10","`npoints'")
               local g "`lbound'"
               local np1 = `npoints' - 1
               forvalue j=2/`np1' {
                  local g "`g' . "
               }
               local g "`g' `ubound'"
            }
            else {
               local g ""
               foreach j of numlist `grid' {
                  local g "`g' `j'"
               }
            }
            if "`jposterior'" != "" local cmd`p' `"`smp`p''(logp,&`logpost'(),`X',`b',`d',"`g'",`maxtry')"'
            else local cmd`p' `"`smp`p''(logp=.,&`logpost'(),`X',`b',`d',"`g'",`maxtry')"'
            local size`p' = 1
            local ++d
         }
      }
*---------------------------
* Skip
*---------------------------
      else if "`smp`p''" == "skip" {
	      local cmd`p' "`smp`p''"
          local size`p' = 1
          local ++d
	  }
*---------------------------
* User's own sampler
*---------------------------
      else {
         if "`jposterior'" != "" local cmd`p' `"`smp`p''(logp,&`logpost'(),`X',`b',`d'"'
         else local cmd`p' `"`smp`p''(logp=.,&`logpost'(),`X',`b',`d'"'
         local 0 `"`prm`p''"'
         syntax , * [ Dim(integer 1) ]
         local size`p' = `dim'
	 tokenize `"`options'"'
         local i = 1
         while "``i''" != "" {
            local cmd`p' "`cmd`p'',``i''"
            local ++i
         }
         local d = `d' + `size`p''
         local cmd`p' "`cmd`p'')"
      }
   }
   
   local npar = `d' - 1
*----------------------------
* Create the MATA program
*----------------------------
   tempname fh
   tempfile matafile
   qui file open `fh' using `matafile' , replace write
   file write `fh' "cap mata : mata drop mcmc()" _n
   file write `fh' "mata:" _n
   file write `fh' "void mcmc() {" _n
   file write `fh' "real scalar fp,j,iter,logp,accept,time2,time3,p" _n
   file write `fh' "string scalar outline" _n
   marksample touse
*----------------------------------
* Transfer data
* transfer b automatically
*----------------------------------
   file write `fh' "real matrix `b' " _n
   file write `fh' `"`b'=st_matrix("`b'")"' _n
   if `"`data'"' != "" {
      tokenize `"`data'"' , parse("=(), ")
      local i = 1
      while "``i''" != "" {
         local j = `i' + 1
         if "``j''" == "=" {
            local ++j
            if "``j''" == "(" {
                local vset ""
                local j = `j' + 1
                while "``j''" != ")" & "``j''" != "" {
                   if "``j''" != "," local vset "`vset' ``j''"
                   local ++j
                }
            }
            else local vset "``j''"
            file write `fh' "real matrix ``i'' " _n
            file write `fh' `"``i''=st_data(.,"`vset'","`touse'")"' _n
            local i = `j' + 1
         }
         else if "``i''" != "`b'" {
            cap confirm matrix ``i''
            if !_rc {
               file write `fh' "real matrix ``i'' " _n
               file write `fh' `"``i''=st_matrix("``i''")"' _n
            }
            cap confirm scalar ``i''
            if !_rc {
               file write `fh' "real scalar ``i'' " _n
               file write `fh' `"``i''=st_numscalar("``i''")"' _n
            }
            local ++i
         }
		 else local ++i
      }
   }
*----------------------------------
* Open file for saving
* tempfiles sometimes left open by Mata when an error occurs
* make attempts to find a fresh filename
*----------------------------------
   if `"`filename'"' != "" {
      tempfile tfile
      cap confirm file "`tfile'"
      local i = 0
      while _rc == 0  & `i' < 20 {
         local ++i
         tempfile tfile
         cap confirm file "`tfile'"
      }
      file write `fh' `"fp=fopen("`tfile'","a")"' _n
      if "`append'" == "append" {
         copy `filename' `tfile' , replace
      }
      else {
         if "`parameters'" == "" {
            if "`savelogp'" != "" local vlist "logp,p1"
		    else local vlist "p1"
            forvalues p = 2/`npar' {
               local vlist "`vlist',p`p'"
            }
         }
         else {
            tokenize "`parameters'" , parse(" -")
            if "`savelogp'" != "" local vlist "logp,`1'"
		    else local vlist "`1'"
            local i = 2
			local h = 1
            while "``i''" != "" {
               if "``i''" == "-" {
                  local j = `i' - 1
                  local t1 "``j''"
                  local ++i
                  local t2 "``i''"
                  local j = strlen("`t1'")
                  while real(substr("`t1'",`j',.)) != . {
                     local --j
                  }
                  local base = substr("`t1'",1,`j')
                  local v1 = substr("`t1'",`j'+1,.)
                  local ++v1
                  local v2 = substr("`t2'",`j'+1,.)
                  forvalues j = `v1'/`v2' {
                     local vlist "`vlist',`base'`j'"
					 local ++h
                  }
               }
               else {
			      local vlist "`vlist',``i''"
				  local ++h
			   }
               local ++i
            }
		    if `h' != `npar' {
di "`vlist'"			
		       di as err "`h' parameter names but `npar' parameters referenced by samplers"
			   exit(499)
		    }
         }
         file write `fh' `"outline = "`vlist'" "' _n
         if `"`predictions'"' != "" {
            file write `fh' "NPD = cols(`predictions'(`X',`b'))" _n
            file write `fh' "for(i=1;i<=NPD;i++) { " _n
            file write `fh' `"outline = outline + "," + "`predictions'" + strofreal(i)"' _n
            file write `fh' "}" _n
         }
         file write `fh' "fput(fp,outline)"  _n
      }
   }
*----------------------------
* Display initial state
*----------------------------
   file write `fh' `"printf("MCMC using `logpost'\n")"' _n
   file write `fh' `"printf("`npar' parameters\n")"' _n
   file write `fh' `"printf("Initial values\n")"' _n
   file write `fh' `"for(j=1;j<=`npar';j++) {"' _n
   file write `fh' `"printf("%10.4f",`b'[j])"' _n
   file write `fh' `"if( mod(j,5) == 0 ) printf("\n") "' _n
   file write `fh' `"}"' _n
   file write `fh' `"printf("\nSamplers\n")"' _n
   local i = 1
   forvalues j = 1/`nSamplers' {
      local ii = `i' - 1 + `size`j''
      if `i' == `ii' local outline : display %4.0f `i' %5s " "
      else local outline : display %4.0f `i' "-" %4.0f `ii'
      local outline2 : display " " %-50s `"`cmd`j''"'
      file write `fh' `"printf(`"`outline' `outline2'\n"')"' _n
      local i = `ii' + 1
   }
   file write `fh' `"printf("Length of Burn-in = `burnin'\n")"' _n
   if "`adapt'" != "" file write `fh' `"printf("Adapting during the burn-in\n")"' _n
   if `dots' > 0 file write `fh' `"printf("dots displayed every `dots' iterations\n")"' _n
   file write `fh' "timer_clear(1)" _n
   file write `fh' "timer_on(1)" _n
   file write `fh' "timer_clear(2)" _n
   file write `fh' "timer_on(2)" _n
   
   file write `fh' "logp=." _n
   file write `fh' "accept = ." _n
   if `burnin' > 0 {
      
      file write `fh' "for(iter = 1;iter<= `burnin';iter++) {" _n
      if `dots'> 0 {
         file write `fh' `"if( mod(iter,`dots') == 0 ) {"' _n
         file write `fh' `" printf(".")"' _n
         file write `fh' "displayflush()" _n
         file write `fh' "}" _n
      }
      forvalues p = 1/`nSamplers' {
	     if "`smp`p''" != "skip" {
            file write `fh' `"`cmd`p''"' _n
            if "`sd`p''" != ""  {
               file write `fh' "if (accept == 1) `sd`p''*1.01" _n
               file write `fh' "else `sd`p''/1.01" _n
            }
		 }
      }
      file write `fh' "}" _n
   }
*----------------------------
* Display state after burn-in
*----------------------------
   file write `fh' "timer_off(2)" _n
   file write `fh' "time2 = timer_value(2)" _n
   if  `burnin' != 0 {
      file write `fh' `"printf("\nBurn-in completed in %10.3f seconds\n",time2[1])"' _n
      file write `fh' `"printf("Parameter values after burn-in\n")"' _n
      file write `fh' `"for(j=1;j<=`npar';j++) {"' _n
      file write `fh' `"printf("%10.4f",`b'[j])"' _n
      file write `fh' `"if( mod(j,5) == 0 ) printf("\n") "' _n
      file write `fh' `"}"' _n
   }
   file write `fh' `"printf("\nRun of length `updates' has started\n")"' _n
   if `"`filename'"' != "" file write `fh' `"printf("Run saved to `filename'\n")"' _n
   if `"`predictions'"' != "" file write `fh' `"printf("Predictions stored as `predictions'1, `predictions'2, ...\n")"' _n
   file write `fh' "timer_clear(3)" _n
   file write `fh' "timer_on(3)" _n
   if `updates' > 0 {
      file write `fh' "for(iter = 1;iter<=`updates';iter++) {" _n
      file write `fh' `"outline = """' _n
      if  `dots' > 0 {
         file write `fh' `"if( mod(iter,`dots') == 0 ) printf(".")"' _n
         file write `fh' "displayflush()" _n
      }
      forvalues p = 1/`nSamplers' {
	     if "`smp`p''" != "skip" {
             file write `fh' `"`cmd`p''"' _n
		 }
      }
      file write `fh' `"outline = strofreal(`b'[1])"' _n
      if `npar' > 1 {
         file write `fh' "for(p=2;p<=`npar';p++) {" _n
         file write `fh' `"outline = outline + "," + strofreal(`b'[p])"' _n
         file write `fh' "}" _n
      }
      if "`savelogp'" != "" file write `fh' `"outline = strofreal(logp) + "," + outline"' _n
      if `"`predictions'"' != "" {
         if `thin' != 1 file write `fh' "if( mod(iter,`thin') == 0 ) {" _n
         file write `fh' "v`predictions' = `predictions'(`X',`b')" _n
         file write `fh' "for(i=1;i<=NPD;i++) { " _n
         file write `fh' `"outline = outline + "," + strofreal(v`predictions'[i])"' _n
         file write `fh' "}" _n
         file write `fh' "fput(fp,outline)"  _n
         if `thin' != 1 file write `fh' "}" _n
      }
      else if `thin' != 1 file write `fh' "if( mod(iter,`thin') == 0 ) fput(fp,outline)" _n
      else file write `fh' "fput(fp,outline)" _n
      file write `fh' "}" _n
   }
   
   file write `fh' "fclose(fp)" _n
   file write `fh' "timer_off(1)" _n
   file write `fh' "timer_off(3)" _n
   file write `fh' "time3 = timer_value(3)" _n
   file write `fh' `"printf("\n `updates' updates completed in %10.3f seconds\n",time3[1])"' _n
   
   if `"`data'"' != "" {
      tokenize `"`data'"' , parse("=() ")
      local i = 1
      while "``i''" != "" {
         local j = `i' + 1
         if "``j''" == "=" {
            while "``j''" != ")" & "``j''" != "" {
               local ++j
            }
            local i = `j' + 1
         }
         else {
            cap confirm matrix ``i''
            if !_rc file write `fh' `"st_matrix("``i''",``i'')"' _n
            cap confirm scalar ``i''
            if !_rc file write `fh' `"st_numscalar("``i''",``i'')"' _n
            local ++i
         }
      }
   }
   
   file write `fh' "}" _n
   file write `fh' "mcmc()" _n
   file write `fh' "end" _n
   file write `fh' "cap mata : mata drop mcmc()" _n
   file close `fh'
   
   di as txt " "
*   di as txt "Mata Program for Running the Analysis"
*   type `matafile'
*   di as txt " "
*   copy `matafile' temp.do   , replace
   do `matafile'
   
*----------------------------
* finish off
*----------------------------
   qui timer list
   copy `tfile' "`filename'" , replace
   return scalar updates = `updates'
   return scalar burnin = `burnin'
   return scalar totaltime = r(t1)
   return scalar burnintime = r(t2)
   return scalar updatetime = r(t3)
end
