*============================================================================
* mcmcsection:  Section plots of an MCMC analysis
* Author:      John Thompson
* Date:        Dec 2012
*============================================================================

program mcmcsection
   version 12.1
   syntax varlist [if] [in] , [ ///
   M(integer 2)                 /// number of sections
   Chain(varname)               /// variable identifying the chain number
   KOPTions(string asis)        /// options passed to kdensity
   GOPTions(string asis)        /// options passed to graph twoway
   CGOPTions(string asis)       /// options passed to graph combine
   NOD                          /// do not show the measure D as a note
   Save(string)                 /// .dta file to save the plotting points
   ]
   
*---------------------------------
* Check options
*---------------------------------
   if `m' < 2 {
      di as err "m must be at least 2"
      exit(499)
   }
*---------------------------------
* Create plot
*---------------------------------
   quietly {
      marksample touse
      preserve
      keep if `touse'
      
*---------------------------------
* count number of parameters
*---------------------------------
      local np = 0
      local np : word count `varlist'
      if `np' > 1 local nodraw "nodraw"
*---------------------------------
* count number of chains
*---------------------------------
      if "`chain'" == "" {
         local nc = 1
      }
      else {
         qui levelsof `chain', local(clist)
         local m : word count `clist'
      }
*---------------------------------
* create order variable
*---------------------------------
      tempvar t h
      if "`chain'" == "" gen `t' = _n
      else {
         gen `h' = _n
         sort `chain' `h'
         by `chain': egen `t' = seq()
      }
*---------------------------------
* file for plotting points
*---------------------------------
      if "`save'" != "" {
         tokenize `"`save'"',parse(",")
         local save = trim(subinstr(`"`1'"',`"""',"",.))
         local dreplace `"`3'"'
      }
*---------------------------------
* plotting points for sections
*---------------------------------
      forvalues i = 1 / `m' {
         tempvar x`i' f`i'
      }
*---------------------------------
* Plot each variable in varlist
*---------------------------------
      local j = 0
      local cs = ""
      tempvar x bigf smallf f plotid
      foreach v of varlist `varlist' {
         local ++j
*-------------------------
* density for the whole chain
*-------------------------
         kdensity `v' , gen(`x' `f') n(101) `koptions' nograph
*-------------------------
* density for each part chain
*-------------------------
         gen `bigf' = 0
         gen `smallf' = 1E10
         local plot "(line `f' `x', clp(solid) clw(medthick) lcol(dknavy))"
         local vars ""
         local lb = 0
         if "`chain'" == "" {
            local size = _N/`m'
            forvalues i = 1 / `m' {
               local ub = `lb' + `size'
               kdensity `v' if `t' > `lb' & `t' <= `ub' , gen(`x`i'' `f`i'') `koptions' at(`x') nograph
               local vars "`vars' `f`i''"
               local plot "`plot' (line `f`i'' `x', clp(dash))"
               local lb = `ub'
               replace `bigf' = `f`i'' if `f`i'' > `bigf'
               replace `smallf' = `f`i'' if `f`i'' < `smallf'
            }
         }
         else {
            local i = 0
            foreach c of local clist {
               local ++i
               kdensity `v' if `chain' == `c' , gen(`x`i'' `f`i'') `koptions' at(`x') nograph
               local vars "`vars' `f`i''"
               local plot "`plot' (line `f`i'' `x', clp(dash) lcol(black))"
               replace `bigf' = `f`i'' if `f`i'' > `bigf'
               replace `smallf' = `f`i'' if `f`i'' < `smallf'
            }
         }
*-------------------------
* Measure of difference
*-------------------------
         summ `f'
         replace `bigf' = (`bigf' - `smallf') / r(max)
         summ `bigf'
         local D = round(r(max) * 100)
*---------------------------------
* files to save plots
*---------------------------------
         tempfile hplot`j'
         local s = "saving(`hplot`j''.gph,replace)"
         local cs  "`cs' `hplot`j''.gph"
         if "`nod'" == "" local note `"note("D=`D'")"'
         else local note ""
         twoway `plot' , legend(off) ytitle("Density") xtitle("`v'") ///
         `note' `s'  `nodraw' `goptions'
         if `"`save'"' != "" {
            tempfile kf`j'
            save "`kf`j''" , replace
         }
         keep `varlist' `t' `chain'
      }
*-------------------------
* combined plot
*-------------------------
      if `np' > 1 {
         graph combine `cs' , `cgoptions'
      }
*-------------------------
* save plotting points
*-------------------------
      if `"`save'"' != "" {
         local j = 0
         foreach v of varlist `varlist' {
            local j = `j' + 1
            if `j' == 1 {
               use "`kf1'" , clear
               tempvar var
               gen str20 `var' = "`v'"
            }
            else {
               append using "`kf`j''"
               replace `var' = "`v'" if `var' == ""
            }
         }
         local klist "x f `var'"
         drop if `f' == .
         rename `f' f
         rename `x' x
         forvalues i=1/`m' {
            local klist "`klist' f`i'"
            rename `f`i'' f`i'
         }
         keep `klist'
         rename `var' parameter
         order parameter
         save `"`save'"' , `dreplace'
      }
      restore
   }
end
