*============================================================================
* mcmcstats:  Summary statistics for an MCMC analysis
* Author:    John Thompson
* Date:      Dec 2012
*============================================================================

program mcmcstats , rclass byable(recall)
   version 12.1
   syntax varlist [if] [in] , [ ///
   Hpd                          /// give HPD rather than credible intervals
   Level(cilevel)               /// % for credible or HPD intervals
   CORrelations                 /// add the correlation matrix to the output
   COVariances                  /// add the covariance matrix to the output
   Save(string)                 /// file to save the table of results
   ]
   
   preserve  
   marksample touse, novarlist
   qui keep if `touse'
*-------------------------------------
* test remaining number obs
*-------------------------------------
   if _N < 2 {
      di as err "Calculation requires at least two iterations"
      restore
      exit(0)
   }
*-------------------------------------
* perform calculations
*-------------------------------------
   tempvar order
   tempname mV C
   cap tsset
   local oldt = r(timevar)
   gen `order' = _n
   qui tsset `order'
*-------------------------------------
* parse save option & open file
*-------------------------------------
   if `"`save'"' != "" {
      tokenize `"`save'"' , parse(" ,")
      tempname pf
      postfile `pf' str20 parameter n mean sd sem median lb ub using `"`1'"', `3'
   }
*--------------------------------
* summarize each variable in turn
*--------------------------------
   di as text _dup(76) "-"
   di as text %-12s "Parameter" %7s "n" %9s "mean" %9s "sd" %9s "sem" %9s "median" _continue
   if "`hpd'" == "" di as text %16s "`level'% CrI"
   else di as text %16s "`level'% HPD"
   di as text _dup(76) "-"
   local lcentile = (100-`level')/2
   local ucentile = 100-`lcentile'
   local i = 0
   foreach v of varlist `varlist' {
*---------------------------------------
* Calculated the statistics
*---------------------------------------
      qui summ `v' 
      local mn = r(mean)
      local sd = r(sd)
      local n = r(N)
      qui centile `v' 
      local md = r(c_1)
      local lb "."
      local ub "."
      if "`hpd'" == "" & `n' > 5 {
         qui centile `v'  , cen(`lcentile')
         local lb = r(c_1)
         qui centile `v'  , cen(`ucentile')
         local ub = r(c_1)
      }
      else if `n' > 5 {
         mcmcstats_hpd `v'  , alpha(`level')
         local lb = r(low)
         local ub = r(high)
      }
      qui cap prais `v' 
      if _rc == 0 {
         matrix `mV' = e(V)
         local se = sqrt(`mV'[1 , 1])
      }
      else local se "."
      di as text " " %-12s "`v'" _continue
      di as res %6.0f `n' " " %8.3f `mn' " " %8.3f `sd' " " %8.4f `se' " " %8.3f `md' " (" %8.3f `lb' "," %8.3f `ub' " )"
      local i = `i' + 1
      return local par`i' "`v'"
      return scalar n`i' = `n'
      return scalar mn`i' = `mn'
      return scalar sd`i' = `sd'
      return scalar se`i' = `se'
      return scalar md`i' = `md'
      return scalar lb`i' = `lb'
      return scalar ub`i' = `ub'
      if "`save'" != "" {
         post `pf' ("`v'") (`n') (`mn') (`sd') (`se') (`md') (`lb') (`ub')
      }
   }
   di as text _dup(76) "-"
*-------------------------------------
* add correlations & covariances
*-------------------------------------
   if "`covariances'" != "" {
      di as txt _newline "Covariances"
      corr `varlist' `if' `in', cov
      matrix `C' = r(C)
      return matrix V = `C'
   }
   if "`correlations'" != "" {
      di as txt _newline "Correlations"
      corr `varlist' `if' `in'
      matrix `C' = r(C)
      return matrix C = `C'
   }
*-------------------------------------
* close save file
*-------------------------------------
   if "`save'" != "" {
      postclose `pf'
   }
   cap tsset `oldt'
   restore
end

*-------------------------------------
* Approximate HPD interval
*-------------------------------------
program mcmcstats_hpd , rclass
   syntax varlist (max = 1 min = 1) [if] [in] [ , alpha(real 95.0) ]
   version 12.1
   
   quietly {
      marksample touse
      local theta = "`varlist'"
      
*--------------------------------
* Initial values based on credible interval
*--------------------------------
      local a = 100 - `alpha'
      local a1 = `a' / 2
      centile `theta'  , centile(`a1' 50 )
      local theta1 = r(c_1)
      local thetam = r(c_2)    
*--------------------------------
* smoothed density of theta in (x,f)
*--------------------------------    
      preserve
      tempvar x f cf tmp
      local n = 1000
      if `n' > _N {
         set obs `n'
      }
      kdensity `theta'  , gen(`x' `f') nograph n(`n')
      gen `cf' = sum(`f')
      su `cf'
      replace `cf' = `cf' / r(max)
      su `x'
      local step = (r(max)-r(min)) / 100
      local xthreshold = 0.001*`step' 
      local athreshold = 0.0001  
*--------------------------------
* Iterate
*--------------------------------   
      if `step' <= 1e-7 {
         local xlow "."
         local xhigh "."
         local a1 "."
         local a2 "."
      }
      else {
      local xlow = `theta1'
      gen `tmp' = .
      local done = 0
      while `done' == 0 & `step' > `xthreshold' {
         * area to left of xlow
         mcmcstats_interp `x' `cf'  , value(`xlow')
         local a1 = r(fvalue)
         * xhigh corresponding to xlow
         mcmcstats_interp `x' `f' if `x' < `thetam' , value(`xlow')
         local fx = r(fvalue)
         mcmcstats_interp `f' `x' if `x' > `thetam' , value(`fx') down
         local xhigh = r(fvalue)
         * area to right of xlow
         mcmcstats_interp `x' `cf'  , value(`xhigh')
         local a2 = 1 - r(fvalue)
         local asum = (`a1' + `a2') * 100
         if abs(`asum'-`a') < `athreshold' {
            local done = 1
         }
         else if `asum' > `a' {
            local xlow = `xlow' - `step'
         }
         else {
            local xlow = `xlow' + `step'
         }
         local step = `step' * 0.9
      }
      if `xhigh' == . {
         local xlow "."
         local a1 "."
         local a2 "."
      }
      }
      restore
      return scalar low = `xlow'
      return scalar high = `xhigh'
      return scalar larea = `a1' * 100
      return scalar rarea = `a2' * 100
   }
end

*-------------------------------------
* Interpolations
*-------------------------------------
program mcmcstats_interp , rclass
   syntax varlist (min = 2 max = 2) [if] , value(real) [ down ]
   version 12.1
   marksample touse
   tokenize "`varlist'"

   local x = "`1'"
   local f = "`2'"
   tempvar p
   gen `p' = _n
   if "`down'" == "" {
      su `p' if `x' < `value' & `touse'
   }
   else {
      su `p' if `x' > `value' & `touse'
   }
   local p1 = r(max)
   local x1 = `x'[`p1']
   local f1 = `f'[`p1']
   local p1 = `p1' + 1
   local x2 = `x'[`p1']
   local f2 = `f'[`p1']
   return scalar fvalue = `f1' + (`value'-`x1') * (`f2'-`f1') / (`x2'-`x1')
end
