*------------------------------
* Code for Chapter 11
* working directory must include the data file: oman.dta
*------------------------------
clear
set more off
set scheme sj
program drop _all
pause off

*==================================================================
* Case Study 12: creatine clearance
*==================================================================

use data/oman.dta, clear
*--------------------------------------------------
* Subject 61 is an outlier and was repmoved in the
* original paper 
*--------------------------------------------------
replace id = id + 61*(sex==2)
drop if id == 61
replace id = id - 1 if id > 61
*--------------------------------------------------
* Put onto a log scale & plot: figure 11.4
*--------------------------------------------------
gen x_1 = log10(MCC)
gen x_2 = log10(ECC)
twoway (scatter x_2 x_1 ) (function y=x, range(1.25 2.5)), ///
  leg(off) xtitle(log{subscript:10}(MCC)) ytitle(log{subscript:10}(ECC)) ///
  xscale( range(1.25 2.5) ) yscale( range(1.25 2.5) ) ///
  xlabel(1.25(0.25)2.5) ylabel(1.25(0.25)2.5)
* FIGURE 11.4  
graph export eps\oman_scatter.eps, replace
*--------------------------------------------------
* Centre the measurements and save
*--------------------------------------------------
replace x_1 = x_1 - 1.85
replace x_2 = x_2 - 1.85
sort id
save temp/omanraw.dta, replace
pause "Press q to continue"

*-------------------------------------------------
* MODEL FIT : Initially without saving the predictions
* in order to check convergence etc
*-------------------------------------------------
tomodel chapter11b.do temp/model.txt
/* 
model {
   for (i in 1:437) {
      x[i,1:2] ~ dmnorm(mu[i,1:2],Omega[1:2,1:2])
      mu[i,1] <- v[id[i]]
      mu[i,2] <- a + b*mu[i,1] + w[id[i]]
   }
   for (i in 1:110) {
      v[i] ~ dnorm(mv,tauv)
      w[i] ~ dnorm(0,tauw)
   }
   Omega[1:2,1:2] ~ dwish(R[1:2,1:2],2)
   tauv ~ dgamma(2,0.02)
   tauw ~ dgamma(2,0.02)   
   a ~ dnorm(0,1)
   b ~ dnorm(1,1)
   mv ~ dnorm(0,1)

# PREDICTIONS FOR MODEL CHECKING

   for (i in 1:110) {
      vstar[i] ~ dnorm(mv,tauv)
      wstar[i] ~ dnorm(0,tauw)
   }
   for (i in 1:437) {
      xstar[i,1:2] ~ dmnorm(mustar[i,1:2],Omega[1:2,1:2])
      bstar[i,1:2] ~ dmnorm(mstar[i,1:2],Omega[1:2,1:2])
      mustar[i,1] <- v[id[i]]
      mustar[i,2] <- a + b*v[id[i]] + w[id[i]]
      mstar[i,1]  <- vstar[id[i]]
      mstar[i,2]  <- a + b*vstar[id[i]] + wstar[id[i]]
   }
}
*/
*--------------------------------------------------
* write the data putting x_1 and x_2 into a
* common structure called x
*--------------------------------------------------
use temp/omanraw.dta, clear
matrix R = 0.02,0.01\ 0.01,0.02
wbslist (vector id,  format(%4.0f)) (matrix R) ///
    (structure x_1 x_2, name(x) format(%8.3f))  using  temp/data.txt, replace
*--------------------------------------------------
* Write initial values & script then run
*--------------------------------------------------
wbslist (Omega=structure(.Data=c(130,-65,-65,130),.Dim=c(2,2)), ///
   mv=0,a=0,b=1,tauw=100,tauv=100) using temp/inits.txt, replace
wbsscript using  temp/script.txt, replace ///
  data(data.txt) model(model.txt) init(inits.txt) ///
  burnin(1000) update(10000) dic logfile(log.txt) path(`c(pwd)'/temp) ///
   set(a b mv tauw tauv Omega ) coda(oman) openbugs
wbsrun using  `c(pwd)'/temp/script.txt, openbugs
type temp/log.txt
*--------------------------------------------------
* Read results and summarise
*--------------------------------------------------
wbscoda using temp/oman, clear openbugs
gen sdw = 1/sqrt(tauw)
gen sdv = 1/sqrt(tauv)
gen sdd = .
gen sde = .
gen rho = .
matrix OMEGA = J(2,2,0)
forvalues i = 1/10000 {
   matrix OMEGA[1,1] = Omega_1_1[`i']
   matrix OMEGA[1,2] = Omega_1_2[`i']
   matrix OMEGA[2,1] = Omega_2_1[`i']
   matrix OMEGA[2,2] = Omega_2_2[`i']
   matrix OMEGA = inv(OMEGA)
   qui replace sdd = sqrt(OMEGA[1,1]) in `i'
   qui replace sde = sqrt(OMEGA[2,2]) in `i'
   qui replace rho = OMEGA[1,2]/(sdd*sde) in `i'
}   
gen taud = 1/(sdd*sdd)
gen taue = 1/(sde*sde)
mcmcstats a b mv sdv sdw sdd sde rho

pause "Press q to continue"

*--------------------------------------------------
* Check convergence
*--------------------------------------------------
mcmctrace a b mv tauw  
pause "Press q to continue"

mcmctrace tauv taud taue rho
pause "Press q to continue"

mcmcac a b mv tauw
pause "Press q to continue"

mcmcac tauv taud taue rho
pause "Press q to continue"

mcmcmahal a b mv tauw tauv taud taue rho, distance(D)
mcmcgeweke D
pause "Press q to continue"

*--------------------------------------------------
* Looks fine to re-run saving 1,000 predictions
*--------------------------------------------------

* remove outlier and centre

use data/oman.dta, clear
replace id = id + 61*(sex==2)
drop if id == 61
replace id = id - 1 if id > 61
gen x_1 = log10(MCC) - 1.85
gen x_2 = log10(ECC) - 1.85
sort id
save temp/omanraw.dta, replace

* Write data, initial values & script then run

matrix R = 0.02,0.01\ 0.01,0.02
wbslist (vector id,  format(%4.0f)) (matrix R) ///
    (structure x_1 x_2, name(x) format(%8.3f))  using  temp/data.txt, replace
wbslist (Omega=structure(.Data=c(130,-65,-65,130),.Dim=c(2,2)), ///
   mv=0,a=0,b=1,tauw=100,tauv=100) using temp/inits.txt, replace
wbsscript using  temp/script.txt, replace ///
  data(data.txt) model(model.txt) init(inits.txt) ///
  burnin(1000) update(10000) thin(10) dic logfile(log.txt)  path(`c(pwd)'/temp)  ///
   set(a b mv tauw tauv Omega xstar bstar ) coda(oman) openbugs
wbsrun using  `c(pwd)'/temp/script.txt, openbugs

* Read the predictions for individual readings

wbscoda using temp/oman, clear openbugs keep(xstar*)

* Recode xstar_17_1 as xstar_1_17 

rename xstar_#_# xs_#[2]_#[1]
save temp/mcmc.dta, replace

* Residual plots

mcmccheck, d(x_1) pred(xs_1_) dfile(temp/omanraw.dta) pfile(temp/mcmc.dta) ///
  plot(summary) cgopt(title(MCC)) 
mcmccheck, d(x_2) pred(xs_2_) dfile(temp/omanraw.dta) pfile(temp/mcmc.dta) ///
  plot(summary) cgopt(title(ECC)) 

* read predictions for subject means & recode

wbscoda using temp/oman, clear openbugs keep(bstar*)
rename bstar_#_# bs_#[2]_#[1]
save temp/mcmcb.dta, replace

* Observed subject means

use temp/omanraw.dta, clear
sort id
by id : egen mx_1 = mean(x_1)
by id : egen mx_2 = mean(x_2)
keep if id != id[_n-1]
save temp/omanmean.dta, replace

* Corresponding mean predictions

use temp/omanraw.dta, clear
forvalues i=1/437 {
  local id`i' = id[`i']
}  
use temp/mcmcb.dta, clear
forvalues j=1/110 {
  local vlist1 ""
  local vlist2 ""
  forvalues i=1/437 {
     if `id`i'' == `j' {
       local vlist1 "`vlist1' bs_1_`i'"
        local vlist2 "`vlist2' bs_2_`i'"
    }
  }
  egen mbs_1_`j' = rmean(`vlist1')
  egen mbs_2_`j' = rmean(`vlist2')
}
keep mbs*
save temp/mcmcb.dta, replace

* Residual plots

mcmccheck, d(mx_1) pred(mbs_1_) dfile(temp/omanmean.dta) pfile(temp/mcmcb.dta) ///
  plot(summary) cgopt(title(MCC)) 
mcmccheck, d(mx_2) pred(mbs_2_) dfile(temp/omanmean.dta) pfile(temp/mcmcb.dta) ///
  plot(summary) cgopt(title(ECC)) 

*--------------------------------------------------
* plots for the book
*--------------------------------------------------
mcmccheck, d(x_2) pred(xs_2_) dfile(temp/omanraw.dta) pfile(temp/mcmc.dta) ///
  plot(summary) 
* FIGURE 11.5  
graph export eps\oman_summary_y.eps, replace
pause "Press q to continue"

mcmccheck, d(mx_2) pred(mbs_2_) dfile(temp/omanmean.dta) pfile(temp/mcmcb.dta) ///
  plot(summary) 
* FIGURE 11.5  
graph export eps\oman_summary_my.eps, replace
pause "Press q to continue"

*--------------------------------------------------
* SECOND MODEL : mv and regression coef (a,b) depend on group
*--------------------------------------------------
tomodel chapter11b.do temp/model2.txt
/* 
model {
   for (i in 1:437 ) {
      x[i,1:2] ~ dmnorm(mu[i,1:2],Omega[1:2,1:2])
     mu[i,1] <- v[id[i]]
     mu[i,2] <- a[gp[i]] + b[gp[i]]*mu[i,1] + c[id[i]]
   }
   for (i in 1:110) {
      v[i] ~ dnorm(mv[g[i]],tauv)
     c[i] ~ dnorm(0,tauc)
   }
   Omega[1:2,1:2] ~ dwish(R[1:2,1:2],2)

   tauv ~ dgamma(2,0.02)
   tauc ~ dgamma(2,0.02)   

   for (j in 1:4) {
      a[j] ~ dnorm(0,1)
      b[j] ~ dnorm(1,1)
      mv[j] ~ dnorm(0,1)
   }

   for (i in 1:110) {
      vstar[i] ~ dnorm(mv[g[i]],tauv)
     cstar[i] ~ dnorm(0,tauc)
   }
   for (i in 1:437) {
      xstar[i,1:2] ~ dmnorm(mustar[i,1:2],Omega[1:2,1:2])
      bstar[i,1:2] ~ dmnorm(mstar[i,1:2],Omega[1:2,1:2])
     mustar[i,1] <- v[id[i]]
     mustar[i,2] <- a[gp[i]] + b[gp[i]]*v[id[i]] + c[id[i]]
     mstar[i,1] <- vstar[id[i]]
     mstar[i,2] <- a[gp[i]] + b[gp[i]]*vstar[id[i]] + cstar[id[i]]
   }
}
*/
use data/oman.dta, clear
replace id = id + 61*(sex==2)
drop if id == 61
replace id = id - 1 if id > 61

gen x_1 = log10(MCC)-1.85
gen x_2 = log10(ECC)-1.85
replace sex = sex == 1
gen gp = sex*2 + fluid + 1
table gp, c(mean sex mean fluid n x_1 mean x_1 mean x_2)
gen g = .
forvalues i=1/110 {
   qui summarize gp if id == `i'
   qui replace g = r(mean) in `i'
}
save temp/oman2raw.dta, replace

matrix R = 0.02,0.01\ 0.01,0.02
qui wbslist (vector id gp g,  format(%4.0f) ) (matrix R) ///
 (structure x_1 x_2, name(x) format(%8.3f) ) ///
  using  temp/data.txt, replace

qui wbslist (Omega=structure(.Data=c(130,-65,-65,130),.Dim=c(2,2)), ///
  mv=c(4{0}),a=c(4{0}),b=c(4{1}),tauc=100,tauv=100) using temp/inits.txt, replace
wbsscript using  temp/script.txt, replace ///
  data(data.txt) model(model2.txt) init(inits.txt) ///
  burnin(1000) update(10000)  dic logfile(log2.txt) path(`c(pwd)'/temp)  ///
  set(a b mv tauc tauv Omega xstar bstar) coda(oman2) openbugs 
  
wbsrun using  `c(pwd)'/temp/script.txt, openbugs
type temp/log2.txt

wbscoda using temp/oman2, clear openbugs keep( a[* b[* mv* tauc tauv Omega* )
gen sc = 1/sqrt(tauc)
gen sv = 1/sqrt(tauv)
gen sd = .
gen se = .
gen rho = .
matrix OMEGA = J(2,2,0)
forvalues i = 1/10000 {
   matrix OMEGA[1,1] = Omega_1_1[`i']
   matrix OMEGA[1,2] = Omega_1_2[`i']
   matrix OMEGA[2,1] = Omega_2_1[`i']
   matrix OMEGA[2,2] = Omega_2_2[`i']
   matrix OMEGA = inv(OMEGA)
   qui replace sd = sqrt(OMEGA[1,1]) in `i'
   qui replace se = sqrt(OMEGA[2,2]) in `i'
   qui replace rho = OMEGA[1,2]/(sd*se) in `i'
}   
gen taud = 1/(sd*sd)
gen taue = 1/(se*se)
mcmcstats a_* b_* mv* sv sd sc se rho taud taue
pause "Press q to continue"

*--------------------------------------------------
* Check convergence
*--------------------------------------------------
mcmctrace a_*
pause "Press q to continue"

mcmctrace b_*
pause "Press q to continue"

mcmctrace mv*  
pause "Press q to continue"

mcmctrace tauc tauv taud taue
pause "Press q to continue"

mcmcmahal a_* b_* mv* tauc tauv taud taue rho, distance(D)
mcmcgeweke D
pause "Press q to continue"

*--------------------------------------------------
* Read predictions xstar
*--------------------------------------------------
wbsscript using  temp/script.txt, replace ///
  data(data.txt) model(model2.txt) init(inits.txt) ///
  burnin(1000) update(10000) thin(10) dic  path(`c(pwd)'/temp)  ///
  set(a b mv tauc tauv Omega xstar bstar) coda(oman2) openbugs 
  
wbsrun using  `c(pwd)'/temp/script.txt, openbugs

wbscoda using temp/oman2, clear openbugs keep(xstar*)
rename xstar_#_# xs_#[2]_#[1]
save temp/mcmc2.dta, replace
pause "Press q to continue"

*--------------------------------------------------
* Individual reading level summary plots for MCC & ECC
*--------------------------------------------------
mcmccheck, d(x_1) pred(xs_1_) dfile(temp/oman2raw.dta) pfile(temp/mcmc2.dta) ///
  plot(summary)   cgopt(title(MCC)) save(temp/resid2x.dta,replace)
pause "Press q to continue"

mcmccheck, d(x_2) pred(xs_2_) dfile(temp/oman2raw.dta) pfile(temp/mcmc2.dta) ///
  plot(summary)   cgopt(title(ECC)) save(temp/resid2y.dta,replace)
pause "Press q to continue"

*--------------------------------------------------
* THIRD MODEL : mv depends on group but reg coef fixed
*--------------------------------------------------
tomodel chapter11b.do temp/model3.txt
/*
model {
   for (i in 1:437) {
      x[i,1:2] ~ dmnorm(mu[i,1:2],Omega[1:2,1:2])
     mu[i,1] <- v[id[i]]
     mu[i,2] <- a + b*mu[i,1] + c[id[i]]
   }
   for (i in 1:110) {
      v[i] ~ dnorm(mv[g[i]],tauv)
     c[i] ~ dnorm(0,tauc)
   }
   Omega[1:2,1:2] ~ dwish(R[1:2,1:2],2)

   tauv ~ dgamma(2,0.02)
   tauc ~ dgamma(2,0.02)   
   a ~ dnorm(0,1)
   b ~ dnorm(0,1)
   
   for (j in 1:4) {
      mv[j] ~ dnorm(0,1)
   }

   for (i in 1:110) {
      vstar[i] ~ dnorm(mv[g[i]],tauv)
     cstar[i] ~ dnorm(0,tauc)
   }
   for (i in 1:437) {
      xstar[i,1:2] ~ dmnorm(mustar[i,1:2],Omega[1:2,1:2])
      bstar[i,1:2] ~ dmnorm(mstar[i,1:2],Omega[1:2,1:2])
     mustar[i,1] <- v[id[i]]
     mustar[i,2] <- a + b*v[id[i]] + c[id[i]]
     mstar[i,1] <- vstar[id[i]]
     mstar[i,2] <- a + b*vstar[id[i]] + cstar[id[i]]
   }
}
*/
use data/oman.dta, clear

replace id = id + 61*(sex==2)
drop if id == 61
replace id = id - 1 if id > 61

gen x_1 = log10(MCC)-1.85
gen x_2 = log10(ECC)-1.85
replace sex = sex == 1
gen gp = sex*2 + fluid + 1
table gp, c(mean sex mean fluid n x_1 mean x_1 mean x_2)
gen g = .
forvalues i=1/110 {
   qui summarize gp if id == `i'
   qui replace g = r(mean) in `i'
}
save temp/oman3raw.dta, replace

matrix R = 0.02,0.01\ 0.01,0.02
qui wbslist (vector id g,  format(%4.0f) ) (matrix R) ///
 (structure x_1 x_2, name(x) format(%8.3f) ) ///
  using  temp/data.txt, replace

qui wbslist ( tauc=100,tauv=100,a=0,b=1,Omega=structure(.Data=c(130,-65,-65,130),.Dim=c(2,2)) ///
  ,mv=c(4{0})) using temp/inits.txt, replace

wbsscript using  temp/script.txt, replace ///
  data(data.txt) model(model3.txt) init(inits.txt) ///
  burnin(1000) update(10000) dic logfile(log3.txt)  path(`c(pwd)'/temp) ///
  set(a b mv tauc tauv Omega) coda(oman3) openbugs 
  
wbsrun using  `c(pwd)'/temp/script.txt, openbugs
type temp/log3.txt

wbscoda using temp/oman3, clear openbugs keep( a b mv* tauc tauv Omega* )
gen sc = 1/sqrt(tauc)
gen sv = 1/sqrt(tauv)
gen sd = .
gen se = .
gen rho = .
matrix OMEGA = J(2,2,0)
forvalues i = 1/10000 {
   matrix OMEGA[1,1] = Omega_1_1[`i']
   matrix OMEGA[1,2] = Omega_1_2[`i']
   matrix OMEGA[2,1] = Omega_2_1[`i']
   matrix OMEGA[2,2] = Omega_2_2[`i']
   matrix OMEGA = inv(OMEGA)
   qui replace sd = sqrt(OMEGA[1,1]) in `i'
   qui replace se = sqrt(OMEGA[2,2]) in `i'
   qui replace rho = OMEGA[1,2]/(sd*se) in `i'
}   
gen taud = 1/(sd*sd)
gen taue = 1/(se*se)
mcmcstats a b mv* sv sd sc se rho taud taue
pause "Press q to continue"

*--------------------------------------------------
* Check convergence
*--------------------------------------------------
mcmctrace a b taud taue
pause "Press q to continue"

mcmctrace mv*  
pause "Press q to continue"

mcmcmahal a b mv* taud taue rho, distance(D)
mcmcgeweke D
pause "Press q to continue"

*--------------------------------------------------
* Read predictions xstar
*--------------------------------------------------
wbsscript using  temp/script.txt, replace ///
  data(data.txt) model(model3.txt) init(inits.txt) ///
  burnin(1000) update(10000) thin(10) dic  path(`c(pwd)'/temp)  ///
  set(xstar) coda(oman3) openbugs 
  
wbsrun using  `c(pwd)'/temp/script.txt, openbugs

wbscoda using temp/oman3, clear openbugs keep(xstar*)
rename xstar_#_# xs_#[2]_#[1]
save temp/mcmc3.dta, replace
pause "Press q to continue"

*--------------------------------------------------
* Individual reading level summary plots for MCC & ECC
*--------------------------------------------------
mcmccheck, d(x_1) pred(xs_1_) dfile(temp/oman3raw.dta) pfile(temp/mcmc3.dta) ///
  plot(summary)   cgopt(title(MCC)) save(temp/resid3x.dta,replace)
pause "Press q to continue"

mcmccheck, d(x_2) pred(xs_2_) dfile(temp/oman3raw.dta) pfile(temp/mcmc3.dta) ///
  plot(summary)   cgopt(title(ECC)) save(temp/resid3y.dta,replace)

pause "Press q to continue"

