*------------------------------
* Code for Chapter 11
* working directory must include the data file: gene.dta
*------------------------------
clear
set more off
set scheme sj
program drop _all
pause off

*==================================================================
* Case Study 12: microarray
*==================================================================

use data/gene.dta, clear
*-------------------------------------------
* Histogram of precisions
*-------------------------------------------
egen m = rmean(logratio1-logratio6)
egen s = rsd(logratio1-logratio6)
gen prec = 1/(s*s)
twoway (histogram prec if prec < 500, bin(50)) ///
   (function y=gammaden(1.5,40,0,x), range(0 500)), leg(off) ///
   xtitle(Precision) ytitle(" ") ylabel(none)
* FIGURE 11.7  
graph export eps\delmar_prec.eps, replace
pause "Press q to continue"

*-------------------------------------------
* Histogram of means
*-------------------------------------------
twoway (histogram m if abs(m) < 0.75, bin(50)) ///
   (function y=normalden(x,0,0.08), range(-0.75 0.75)), leg(off) ///
   xtitle(Mean) ytitle(" ") ylabel(none) xscale( range(-0.75 0.75)) ///
   xlabel(-0.75(0.25)0.75)
* FIGURE 11.8  
graph export eps\delmar_mean.eps, replace
pause "Press q to continue"

*-------------------------------------------
* t-statistics against expected t
*-------------------------------------------
use data/gene.dta, clear
*-------------------------------------------
* Histogram of precisions
*-------------------------------------------
egen m = rmean(logratio1-logratio6)
egen s = rsd(logratio1-logratio6)
gen se = s/sqrt(6)
gen t = m/se
gen p = 2*ttail(5,abs(t))
sort t
gen z = invttail(5,1-_n/(_N+1))
twoway (scatter t z) (function y=x, range(-10 10)), leg(off) ytitle(t-statistic) xtitle(Expected value)
* FIGURE 11.6  
graph export eps\delmar_t.eps, replace
pause "Press q to continue"

*-------------------------------------------
* Gamma prior on precisions
*-------------------------------------------
tomodel chapter11c.do temp/models.txt
/* 
model {
   for (i in 1:4360) {
      mu[i] ~ dnorm(0,4)
      prec[i] ~ dgamma(1.5,0.025)
      for (j in 1:6) {
         y[i,j] ~ dnorm(mu[i],prec[i])
      }
      t[i] <- mu[i]*sqrt(6*prec[i])
   }
}   
*/
use data/gene.dta, clear
qui wbslist (structure logratio1-logratio6, name(y) format(%6.3f))  using temp/data.txt, replace
qui wbslist (mu=c(4360{0}),prec=c(4360{100})) using temp/inits.txt, replace
wbsscript using temp/script.txt, replace ///
  data(data.txt) model(models.txt) init(inits.txt) ///
  burnin(1000) update(10000) thin(10) dic logfile(logs.txt) ///
  set(t) coda(delmars) openbugs path(`c(pwd)'/temp)
wbsrun using `c(pwd)'/temp/script.txt, openbugs
wbscoda using temp/delmars, clear openbugs 
save temp/delmarsmcmc.dta, replace
pause "Press q to continue"

*-------------------------------------------
* Convergence checking
* 4360 genes so a token effort
* but it is a simple model
*-------------------------------------------
mcmctrace t_500 t_1500 t_2500 t_3500
pause "Press q to continue"

*-------------------------------------------
* Summarize the chain & plot posterior means
*-------------------------------------------
qui mcmcstats t_*, save(temp/delmars.dta,replace)
use temp/delmars.dta, clear
sort mean
gen z = invttail(5,1-_n/4361)
twoway (scatter mean z) (function y=x, range(-10 10)), leg(off) ///
    ytitle(t-statistic) xtitle(Expected value)  
pause "Press q to continue"

*-------------------------------------------
* MODELM.TXT: MIXTURE FOR MU
* TAKES LONGER TO RUN - CLEAR SPLIT BETWEEN NULL & EXPRESSED GENES
*-------------------------------------------
tomodel chapter11c.do temp/modelm.txt
/*
 model {
   for (i in 1:4360) {
      d[i] ~ dbern(p)
     s[i] ~ dbern(0.5)
     m[i] ~ dnorm(1.5,4)
     mu[i] <- d[i]*(1-2*s[i])*m[i]
     prec[i] ~ dgamma(1.5,0.025)
      for (j in 1:6) {
        y[i,j] ~ dnorm(mu[i],prec[i])
     }
     t[i] <- mu[i]*sqrt(6*prec[i])
   }
   p ~ dbeta(1,50)
} 
*/

use data/gene.dta, clear
wbslist (structure logratio1-logratio6, name(y) format(%6.3f))  ///
    using temp/data.txt, replace
wbslist (prec=c(4360{100}),p=0.02,s=c(4360{runiform()<0.5}),m=c(4360{1.5}), ///
   d=c(4360{runiform()<0.02})) using temp/inits.txt, replace
wbsscript using temp/script.txt, replace ///
  data(data.txt) model(modelm.txt) init(inits.txt) ///
  burnin(1000) update(10000) thin(10) dic logfile(logm.txt) ///
  set(t p) coda(delmarm) openbugs path(`c(pwd)'/temp)
wbsrun using `c(pwd)'/temp/script.txt, openbugs
wbscoda using temp/delmarm, clear openbugs
save temp/delmarmmcmc.dta, replace
mcmcstats p
pause "Press q to continue"
*-------------------------------------------
* Summarize the chain & plot posterior means
*-------------------------------------------
qui mcmcstats t_*, save(temp/delmarm.dta,replace)
use temp/delmarm.dta, clear
sort mean
histogram mean, normal bin(50)
gen z = invttail(5,1-_n/4361)
twoway (scatter mean z) (function y=x, range(-10 10)), leg(off) ytitle(t-statistic) xtitle(Expected value)  
graph export eps\delmar_bayesm.eps, replace
pause "Press q to continue"

*-------------------------------------------
* Mixture models with 2 to 5 components
*-------------------------------------------
tomodel chapter11c.do temp/model2.txt
/*
model {
   for (i in 1:4360) {
      set[i] ~ dcat(p[1:2])
     mu[i] ~ dnorm(0,0.001)
      for (j in 1:6) {
        y[i,j] ~ dnorm(mu[i],prec[set[i]])
     }
     t[i] <- mu[i]*sqrt(6*prec[set[i]])
   }
   prec[1] ~ dgamma(1.5,0.025) C(,prec[2])
   prec[2] ~ dgamma(1.5,0.025) C(prec[1],)
   p[1:2] ~ ddirich(alpha[1:2])
   for (i in 1:4360) {
      for (j in 1:6) {
        LL[i,j] <- 0.5*log(prec[set[i]])-0.5*prec[set[i]]*(y[i,j]-mu[i])*
            (y[i,j]-mu[i])
     }
   }
   logL <- sum(LL[,])
   keep[1] <- mu[2090]
   keep[2] <- prec[set[2090]]
   keep[3] <- t[2090]
   keep[4] <- set[2090]
}  
*/
tomodel chapter11c.do temp/model3.txt
/*
model {
   for (i in 1:4360) {
      set[i] ~ dcat(p[1:3])
     mu[i] ~ dnorm(0,0.001)
      for (j in 1:6) {
        y[i,j] ~ dnorm(mu[i],prec[set[i]])
     }
     t[i] <- mu[i]*sqrt(6*prec[set[i]])
   }
   prec[1] ~ dgamma(1.5,0.025) C(,prec[2])
   prec[2] ~ dgamma(1.5,0.025) C(prec[1],prec[3])
   prec[3] ~ dgamma(1.5,0.025) C(prec[2],)
   p[1:3] ~ ddirich(alpha[1:3])

# log likelihood

   for (i in 1:4360) {
      for (j in 1:6) {
         LL[i,j] <- 0.5*log(prec[set[i]]) -
                    0.5*prec[set[i]]*(y[i,j]-mu[i])*(y[i,j]-mu[i])
      }
   }
   logL <- sum(LL[,])

# results for gene p21

   keep[1] <- mu[2090]
   keep[2] <- prec[set[2090]]
   keep[3] <- t[2090]
   keep[4] <- set[2090]
}   
*/
tomodel chapter11c.do temp/model4.txt
/*
model {
   for (i in 1:4360) {
      set[i] ~ dcat(p[1:4])
     mu[i] ~ dnorm(0,0.001)
      for (j in 1:6) {
        y[i,j] ~ dnorm(mu[i],prec[set[i]])
     }
     t[i] <- mu[i]*sqrt(6*prec[set[i]])
   }
   prec[1] ~ dgamma(1.5,0.025) C(,prec[2])
   prec[2] ~ dgamma(1.5,0.025) C(prec[1],prec[3])
   prec[3] ~ dgamma(1.5,0.025) C(prec[2],prec[4])
   prec[4] ~ dgamma(1.5,0.025) C(prec[3],)
   p[1:4] ~ ddirich(alpha[1:4])
   for (i in 1:4360) {
      for (j in 1:6) {
        LL[i,j] <- 0.5*log(prec[set[i]])-0.5*prec[set[i]]*(y[i,j]-mu[i])*(y[i,j]-mu[i])
     }
   }
   logL <- sum(LL[,])
   keep[1] <- mu[2090]
   keep[2] <- prec[set[2090]]
   keep[3] <- t[2090]
   keep[4] <- set[2090]
}   
*/
tomodel chapter11c.do temp/model5.txt
/*
model {
   for (i in 1:4360) {
      set[i] ~ dcat(p[1:5])
     mu[i] ~ dnorm(0,0.001)
      for (j in 1:6) {
        y[i,j] ~ dnorm(mu[i],prec[set[i]])
     }
     t[i] <- mu[i]*sqrt(6*prec[set[i]])
   }
   prec[1] ~ dgamma(1.5,0.025) C(,prec[2])
   prec[2] ~ dgamma(1.5,0.025) C(prec[1],prec[3])
   prec[3] ~ dgamma(1.5,0.025) C(prec[2],prec[4])
   prec[4] ~ dgamma(1.5,0.025) C(prec[3],prec[5])
   prec[5] ~ dgamma(1.5,0.025) C(prec[4],)
   p[1:5] ~ ddirich(alpha[1:5])
   for (i in 1:4360) {
      for (j in 1:6) {
        LL[i,j] <- 0.5*log(prec[set[i]])-0.5*prec[set[i]]*(y[i,j]-mu[i])*(y[i,j]-mu[i])
     }
   }
   logL <- sum(LL[,])
   keep[1] <- mu[2090]
   keep[2] <- prec[set[2090]]
   keep[3] <- t[2090]
   keep[4] <- set[2090]
}   
*/
*-------------------------------------------------------------------
* VARIANCE MIXTURE AS PROPOSED BY DELMAR: MODEL2.TXT
* THIS MIXTURE FOR THE VARIANCE HAS TWO COMPONENTS
* keep is variable in the model that stored data about GENE 2090
*-------------------------------------------------------------------
use data/gene.dta, clear
qui wbslist (alpha=c(1,1)) (structure logratio1-logratio6, name(y) format(%6.3f)) ///
    using temp/data.txt, replace
qui wbslist (prec=c(50,100),mu=c(4360{0}),set=c(4360{1+(runiform()<0.3)})) ///
    using temp/inits.txt, replace
wbsscript using temp/script.txt, replace ///
  data(data.txt) model(model2.txt) init(inits.txt) ///
  burnin(1000) update(10000) thin(10) logfile(log2.txt) ///
  set(prec p keep logL) coda(delmar2) openbugs path(`c(pwd)'/temp)
wbsrun using `c(pwd)'/temp/script.txt, openbugs
wbscoda using temp/delmar2, clear openbugs
save temp/delmar2mcmc.dta, replace
forvalues i=1/2 {
   gen sd_`i' = 1/sqrt(prec_`i')
}
mcmcstats p* s* logL k*
*-------------------------------------------------------------------
* VARIANCE MIXTURE AS PROPOSED BY DELMAR: MODEL3.TXT
* THIS MIXTURE FOR THE VARIANCE HAS THREE COMPONENTS
* keep is variable in the model that stored data about GENE 2090
*-------------------------------------------------------------------
use data/gene.dta, clear
qui wbslist (alpha=c(1,1,1)) (structure logratio1-logratio6, name(y) format(%6.3f)) ///
   using temp/data.txt, replace
qui wbslist (prec=c(50,100,150),mu=c(4360{0}),set=c(4360{1+(runiform()<0.3)+(runiform()<0.3)})) ///
   using temp/inits.txt, replace
wbsscript using temp/script.txt, replace ///
  data(data.txt) model(model3.txt) init(inits.txt) ///
  burnin(1000) update(10000) thin(10) logfile(log3.txt) ///
  set(prec p keep logL) coda(delmar3) openbugs path(`c(pwd)'/temp)
wbsrun using `c(pwd)'/temp/script.txt, openbugs
wbscoda using temp/delmar3, clear openbugs
save temp/delmar3mcmc.dta, replace
forvalues i=1/3 {
   gen sd_`i' = 1/sqrt(prec_`i')
}
mcmcstats p* s* logL k*
*-------------------------------------------------------------------
* VARIANCE MIXTURE AS PROPOSED BY DELMAR: MODEL4.TXT
* THIS MIXTURE FOR THE VARIANCE HAS FOUR COMPONENTS
* keep is variable in the model that stored data about GENE 2090
*-------------------------------------------------------------------
use data/gene.dta, clear
qui wbslist (alpha=c(1,1,1,1)) (structure logratio1-logratio6, name(y) format(%6.3f))  ///
    using temp/data.txt, replace
qui wbslist (prec=c(50,100,150,200),mu=c(4360{0}),set=c(4360{1+(runiform()<0.3)+(runiform()<0.3)+(runiform()<0.3)})) ///
    using temp/inits.txt, replace
wbsscript using temp/script.txt, replace ///
  data(data.txt) model(model4.txt) init(inits.txt) ///
  burnin(1000) update(10000) thin(10) logfile(log4.txt) ///
  set(prec p keep logL t) coda(delmar4) openbugs path(`c(pwd)'/temp) 
wbsrun using `c(pwd)'/temp/script.txt, openbugs
wbscoda using temp/delmar4, clear openbugs
save temp/delmar4mcmc.dta, replace
forvalues i=1/4 {
   gen sd_`i' = 1/sqrt(prec_`i')
}
mcmcstats p* s* logL k*

use temp/delmar4mcmc.dta, clear
qui mcmcstats t*, save(temp/delmar4t.dta,replace)
*-------------------------------------------------------------------
* VARIANCE MIXTURE AS PROPOSED BY DELMAR: MODEL5.TXT
* THIS MIXTURE FOR THE VARIANCE HAS FIVE COMPONENTS
* keep is variable in the model that stored data about GENE 2090
*-------------------------------------------------------------------
use data/gene.dta, clear
qui wbslist (alpha=c(1,1,1,1,1)) (structure logratio1-logratio6, name(y) format(%6.3f)) ///
   using temp/data.txt, replace
qui wbslist (prec=c(50,100,150,200,250),mu=c(4360{0}),set=c(4360{1+(runiform()<0.3)+(runiform()<0.3)+(runiform()<0.3)+(runiform()<0.3)})) ///
   using temp/inits.txt, replace
wbsscript using temp/script.txt, replace ///
  data(data.txt) model(model5.txt) init(inits.txt) ///
  burnin(1000) update(10000) thin(10) logfile(log5.txt) ///
  set(prec p keep logL) coda(delmar5) openbugs path(`c(pwd)'/temp)
wbsrun using `c(pwd)'/temp/script.txt, openbugs
wbscoda using temp/delmar5, clear openbugs
save temp/delmar5mcmc.dta, replace
forvalues i=1/5 {
   gen sd_`i' = 1/sqrt(prec_`i')
}
mcmcstats p* s* logL k*
*-------------------------------------------------------------------
* plot density of sd of gene 2090 from each model
*-------------------------------------------------------------------
use temp/delmar2mcmc.dta, clear
gen sd = 1/sqrt(keep_2)
mcmcdensity sd, gopt(xtitle(Standard Deviation) xscale(range(0 0.4)) xlabel(0(0.1)0.4))
graph save temp/plot1.gph, replace
use temp/delmar3mcmc.dta, clear
gen sd = 1/sqrt(keep_2)
mcmcdensity sd, gopt(xtitle(Standard Deviation) xscale(range(0 0.4)) xlabel(0(0.1)0.4) )
graph save temp/plot2.gph, replace
use temp/delmar4mcmc.dta, clear
gen sd = 1/sqrt(keep_2)
mcmcdensity sd if sd < 0.4, gopt( xtitle(Standard Deviation) xscale(range(0 0.4)) xlabel(0(0.1)0.4) )
graph save temp/plot3.gph, replace
use temp/delmar5mcmc.dta, clear
gen sd = 1/sqrt(keep_2)
mcmcdensity sd if sd < 0.4, gopt( xtitle(Standard Deviation) xscale(range(0 0.4)) xlabel(0(0.1)0.4) )
graph save temp/plot4.gph, replace
graph combine temp/plot1.gph temp/plot2.gph temp/plot3.gph temp/plot4.gph 
* FIGURE 11.9  
graph export eps\delmar_p21sd.eps, replace

*-------------------------------------------------------------------
* logL for each model
*-------------------------------------------------------------------
use temp/delmar2mcmc.dta, clear
ci logL
use temp/delmar3mcmc.dta, clear
ci logL
use temp/delmar4mcmc.dta, clear
ci logL
use temp/delmar5mcmc.dta, clear
ci logL
summarize p_*

*-------------------------------------------------------------------
* Compare results of modelm and modelt
*-------------------------------------------------------------------
use temp/delmar4t.dta, clear
foreach v of varlist n-ub {
   rename `v' `v'4
}
merge 1:1 _n using temp/delmarm.dta
twoway (scatter mean mean4) (function y=x, range(-10 30) lpat(dash)), ytitle("T-statistic: gamma(1.5,40) model") ///
  xtitle("T-statistic: 4-component mixture model")  yline(0) legend(off)
* FIGURE 11.9  
graph export eps\delmar_twots.eps, replace
