*------------------------------
* Code for Chapter 12
* working directory must include the data file: diabetes.dta
*------------------------------
clear
set more off
set scheme sj
program drop _all
pause off

*------------------------------------
* Laplace lambda = 2
*------------------------------------
clear
range b -5 5 1001 
gen f = exp(-2*abs(b))
line f b , ytitle(Density) ylabel(none) xlabel(-5(1)5)
* FIGURE 12.1  
graph export eps\laplace.eps, replace

*==================================================================
*  MATA code for Bayesian Lasso
*==================================================================
mata:
mata clear
mata set matastrict off
real matrix RIGAUSS(
   real scalar R,
   real scalar C,
   real scalar mu,
   real scalar lambda)
{
    N = rnormal(R,C,0,1)
    Y = N:*N
    X = mu:+0.5:*mu:*(mu:*Y-sqrt(4:*mu:*lambda:*Y:+mu:*mu:*Y:*Y)):/lambda
    U = runiform(R,C):< mu:/(mu:+X)
    V = U:*X :+ (1:-U):*(mu:*mu:/X)
    return(V)
}
void BLASSO(
   string scalar Yname,string scalar Xname,real scalar mmu,real scalar tmu,
   real scalar aphi,real scalar bphi,real scalar aomega,real scalar bomega,
   real scalar burnin,real scalar updates,real scalar thin,
   string scalar filename)
{
   Y = st_data(.,Yname)
   X = st_data(.,Xname)
   N = rows(Y)
   P = cols(X)
/*-----------------------
  Initial Values
-----------------------*/
   SSx = colsum(X:*X)   
   mu = sum(Y)/N
   phi = 1/(Y'*Y/N-mu*mu)
   B = J(P,1,0)
   T = J(P,1,1)
   omega = 1
   FIT = J(N,1,mu) 
/*-----------------------
  iterations
-----------------------*/
   fh = fopen(filename,"w")
   param = "mu,phi,omega"
   param = param + invtokens(",b":+strofreal(range(1,P,1)'))
   param = param + invtokens(",t":+strofreal(range(1,P,1)'))
   fput(fh,param)      
   for (iter=1;iter<=(burnin+updates);iter++) {
   /*-----------------------
    update B
   -----------------------*/
      for (j=1;j<=P;j++) {
         v = B[j]
         prc = T[j] + SSx[j]
         m = X[,j]'*(Y-FIT+v*X[,j])
         B[j] = rnormal(1,1,m/prc,1/sqrt(phi*prc))
         FIT = FIT + (B[j]-v):*X[,j]
      }
   /*-----------------------
    update MU
   -----------------------*/
      v = mu
      m = mmu*tmu + phi*sum(Y-FIT:+v)
      prc = N*phi+tmu
      mu = rnormal(1,1,m/prc,1/sqrt(prc))
      FIT = FIT :+ (mu - v)
   /*-----------------------
    update PHI
   -----------------------*/
      b = 2/bphi + (Y-FIT)'*(Y-FIT) + sum(T:*B:*B)
      a = N/2+P/2+aphi
      phi = rgamma(1,1,a,2/b)
   /*-----------------------
    update OMEGA
   -----------------------*/
      b = 1/bomega + sum(0.5:/T)
      a = P + aomega
      omega = rgamma(1,1,a,1/b)
   /*-----------------------
    update t
   -----------------------*/
      for (j=1;j<=P;j++) T[j] = RIGAUSS(1,1,sqrt(omega/(phi*B[j]*B[j])),omega)
   /*-----------------------
    save estimates
   -----------------------*/
      if( iter > burnin & mod(iter-burnin,thin) == 0 ) {
         theta = mu,phi,omega,B',T'       
         param = strofreal(mu[1]) + 
                  invtokens(",":+strofreal(theta[range(2,3+2*P,1)])) 
         fput(fh,param)
      }
   }
   fclose(fh)
}

end

*==================================================================
*  STATA code for calling the Bayesian Lasso
*==================================================================
program bayeslasso
   syntax varlist [if] [in] using/, MU(numlist max=2 min=2) ///
      PHi(numlist max=2 min=2) OMega(numlist max=2 min=2) ///
     [Burnin(integer 1000) Updates(integer 10000) Thin(integer 1)]

   preserve
   marksample touse     
   qui drop if `touse' == 0
*-------------------------------
* Split Y and X then standardize
*-------------------------------
   local j = 0
   foreach v of varlist `varlist' {
      if `j' == 0 local Y "`v'"
      else {
         tempvar s`j'
         egen `s`j'' = std(`v')
         local xlist "`xlist' `s`j''"
      }
      local ++j
   }
*-------------------------------
* Read the prior parameters
*-------------------------------
   local mmu : word 1 of `mu'
   local tmu : word 2 of `mu'
   local aphi : word 1 of `phi'
   local bphi : word 2 of `phi'
   local aomega : word 1 of `omega'
   local bomega : word 2 of `omega'
*-------------------------------
* Run mata
*-------------------------------
   cap rm `using'
   mata: BLASSO("`Y'","`xlist'",`mmu',`tmu',`aphi',`bphi',`aomega', ///
     `bomega',`burnin',`updates',`thin',"`using'")
   restore
end     

*-------------------------------
* Simulation
*-------------------------------
clear
set obs 200
set seed 450892
forvalues j=1/6 {
   gen x`j' = rnormal()
}
gen y = 5 + x1 + 2*x2 + 2*rnormal()
reg y x1-x6
timer clear 1
timer on 1
bayeslasso y x1-x6 using temp.csv, mu(0 0.0001) phi(0.01 100) omega(50 50) 
timer off 1
timer list 1   
import delimited temp.csv, clear
gen sd = 1/sqrt(phi)
mcmcstats *

*-------------------------------
* Analysis of the Diabetes data
*-------------------------------
use data/diabetes.dta, clear
bayeslasso Y AGE-S6 using temp/temp.csv, ///
     mu(175 0.0004) phi(0.01 0.05) omega(0.04 25) burnin(1000) update(20000)
import delimited temp/temp.csv, clear
gen sd = 1/sqrt(phi)  
gen lambda = sqrt(omega)  
mcmcstats * 
*-------------------------------
* Prior and Posterior of Lambda
*-------------------------------
mcmcdensity lambda , ///
  addplot( function y=2*x*gammaden(0.04,25,0,x^2), range(0.2 15) ///
  lpat(dash) )
* FIGURE 12.2  
graph export eps\lambdaprior.eps, replace
pause "Press q to continue"

*==================================================================
*  MATA code for Bayesian Lasso with fixed lambda (omega)
*==================================================================
mata:
mata clear
mata set matastrict off
real matrix RIGAUSS(
   real scalar R,
   real scalar C,
   real scalar mu,
   real scalar lambda)
{
    N = rnormal(R,C,0,1)
    Y = N:*N
    X = mu:+0.5:*mu:*(mu:*Y-sqrt(4:*mu:*lambda:*Y:+mu:*mu:*Y:*Y)):/lambda
    U = runiform(R,C):< mu:/(mu:+X)
    V = U:*X :+ (1:-U):*(mu:*mu:/X)
    return(V)
}

void BLASSOf(
   string scalar Yname,string scalar Xname,real scalar mmu,real scalar tmu,
   real scalar aphi,real scalar bphi,real scalar omega,
   real scalar burnin,real scalar updates,real scalar thin,string scalar filename)
{

   Y = st_data(.,Yname)
   X = st_data(.,Xname)
   N = rows(Y)
   P = cols(X)
/*-----------------------
  Initial Values
-----------------------*/
   SSx = colsum(X:*X)   
   mu = sum(Y)/N
   phi = 1/(Y'*Y/N-mu*mu)
   B = J(P,1,0)
   T = J(P,1,1)
   FIT = J(N,1,mu) 
/*-----------------------
  iterations
-----------------------*/
   fh = fopen(filename,"w")
  
   param = "mu,phi"
   param = param + invtokens(",b":+strofreal(range(1,P,1)'))
   param = param + invtokens(",t":+strofreal(range(1,P,1)'))
   fput(fh,param)      
   for (iter=1;iter<=(burnin+updates);iter++) {
   /*-----------------------
    update B
   -----------------------*/
      for (j=1;j<=P;j++) {
         v = B[j]
         prc = T[j] + SSx[j]
         m = X[,j]'*(Y-FIT+v*X[,j])
         B[j] = rnormal(1,1,m/prc,1/sqrt(phi*prc))
         FIT = FIT + (B[j]-v):*X[,j]
      }
   /*-----------------------
    update MU
   -----------------------*/
      v = mu
      m = mmu*tmu + phi*sum(Y-FIT:+v)
      prc = N*phi+tmu
      mu = rnormal(1,1,m/prc,1/sqrt(prc))
      FIT = FIT :+ (mu - v)
   /*-----------------------
    update PHI
   -----------------------*/
      b = 2/bphi + (Y-FIT)'*(Y-FIT) + sum(T:*B:*B)
      a = N/2+P/2+aphi
      phi = rgamma(1,1,a,2/b)
   /*-----------------------
    update OMEGA is dropped
   -----------------------*/
   /*-----------------------
    update t
   -----------------------*/
      for (j=1;j<=P;j++) T[j] = RIGAUSS(1,1,sqrt(omega/(phi*B[j]*B[j])),omega)
   /*-----------------------
    save estimates
   -----------------------*/
      if( iter > burnin & mod(iter-burnin,thin) == 0 ) {
         theta = mu,phi,B',T'       
         param = strofreal(mu[1]) + invtokens(",":+strofreal(theta[range(2,2+2*P,1)])) 
         fput(fh,param)
      }
   }
   fclose(fh)

}

end

*==================================================================
*  STATA code for calling the Bayesian Lasso
*==================================================================
program drop _all
program bayeslassof
   syntax varlist [if] [in] using/, MU(numlist max=2 min=2) ///
      PHi(numlist max=2 min=2) OMega(real) ///
     [Burnin(integer 1000) Updates(integer 10000) Thin(integer 1)]

   preserve
   marksample touse     
   qui drop if `touse' == 0
*-------------------------------
* Split Y and X then standardize
*-------------------------------
   local j = 0
   foreach v of varlist `varlist' {
      if `j' == 0 local Y "`v'"
      else {
         tempvar s`j'
         egen `s`j'' = std(`v')
         local xlist "`xlist' `s`j''"
      }
      local ++j
   }
*-------------------------------
* Read the prior parameters
*-------------------------------
   local mmu : word 1 of `mu'
   local tmu : word 2 of `mu'
   local aphi : word 1 of `phi'
   local bphi : word 2 of `phi'
*-------------------------------
* Run mata
*-------------------------------
   cap rm `using'
   mata: BLASSOf("`Y'","`xlist'",`mmu',`tmu',`aphi',`bphi',`omega', ///
     `burnin',`updates',`thin',"`using'")
   restore
end     
  
*--------------------------------------------------
* Analysis of the Diabetes data for fixed lambda
*--------------------------------------------------
tempname pf
postfile `pf' lambda mu age sex bmi map tc ldl hdl tch ltg glu using temp/temp.dta, replace
foreach lambda of numlist 0.1(0.1)0.4 0.5(0.5)10 {
   local omega = `lambda'*`lambda'
   use data/diabetes.dta, clear
   bayeslassof Y AGE-S6 using temp/temp.csv, ///
     mu(175 0.0004) phi(0.01 0.05) omega(`omega') burnin(1000) update(20000)
   import delimited temp/temp.csv, clear
   display %10.2f `omega' %10.2f `lambda'
   mcmcstats mu b* 
   post `pf' (`lambda') (r(mn1)) (r(mn2)) (r(mn3)) (r(mn4)) (r(mn5)) ///
    (r(mn6)) (r(mn7)) (r(mn8)) (r(mn9)) (r(mn10)) (r(mn11))
}
postclose `pf'
use temp/temp.dta, clear
local plot ""
local text ""
local k = 0
foreach v of varlist age sex bmi map tc ldl hdl tch ltg glu {
   local plot "`plot' (line `v' lambda, lpat(solid))"
   local y = `v'[1]
   local x = -0.5
   local ++k
   local text `"`text' text(`y' `x' "`v'")"'
}   
display `"`text'"'
twoway `plot' , `text' leg(off) xscale( range(-1 10)) xlabel(0(2)10) ytitle(Standardized Regression Coefficient)
* FIGURE 12.3  
graph export eps\diabetes1.eps, replace
pause "Press q to continue"

*------------------------------------
* Estimates when lambda = 10
*------------------------------------
use data/diabetes.dta, clear
foreach v of varlist AGE-S6 {
      egen t = std(`v')
     drop `v'
     rename t `v'
}    
   reg Y AGE-S6
   matrix b = e(b)
   local omega = 100
   use data/diabetes.dta, clear
   bayeslassof Y AGE-S6 using temp/temp.csv, ///
     mu(175 0.0004) phi(0.01 0.05) omega(`omega') burnin(1000) update(20000)
   qui import delimited temp/temp.csv, clear
   mcmcstats mu b* , save(temp/lasso10.dta,replace)
use temp/lasso10.dta, clear
gen reg = 0
forvalues i=1/10 {
  local j = `i' + 1
  qui replace reg = b[1,`i'] in `j'   
}
gen id = 11 - _n  
twoway (rcap lb ub id if id <10, horiz) (scatter id mean if id <10, msym(circle)) ///
  (scatter id reg if id <10, msym(x)) , legend(off) ///
  ylabel( 0 "glu" 1 "ltg" 2 "tch" 3 "hdl" 4 "ldl" 5 "tc" 6 "map" 7 "bmi" 8 "sex" 9 "age") ///
  ytitle(" ") xtitle("Standardized Regression Coefficient") xline(0)
* FIGURE 12.4  
graph export eps\diabetes.eps, replace
pause "Press q to continue"
  
